describe Orator::Base do
  it "should call before blocks" do
    something = double('something')
    something.should_receive(:to_call)
    klass = Class.new(described_class) do
      orator_name 'test'
      before { something.to_call }
      on 'some_event'
      def some_event; end
    end

    klass.new(nil).__trigger('test.some_event')
  end

  it "should raise error when it doesn't match the event" do
    expect { described_class.new(nil).__trigger('other.thing') }.to raise_error(NoMethodError)
    expect { described_class.new(nil).__trigger('.thing') }.to raise_error(NoMethodError)
  end

  it "should not run the event if the before callback prevents it" do
    something = double('something')
    something.should_not_receive(:to_call)
    klass = Class.new(described_class) do
      orator_name 'test'
      before { prevent_event }
      on('some_event') { something.to_call }
    end

    klass.new(nil).__trigger('test.some_event')
  end

  it "should register itself with an event handler" do
    event_handler = double('event_handler')

    klass = Class.new(described_class) do
      orator_name 'test'

      on('some_event') { }
    end

    event_handler.should_receive(:on).with("test.some_event", klass)

    klass.register_with(event_handler)
  end

  it "should be able to take a before method" do
    something = double('something')
    something.should_not_receive(:to_call)

    klass = Class.new(described_class) do
      orator_name 'test'

      before :do_something

      def do_something; prevent_event; end

      on 'some_event' do; something.to_call; end
    end

    klass.new(nil).__trigger('test.some_event')
  end

  it "should be able to use a different method" do

    klass = Class.new(described_class) do
      orator_name 'test'

      on 'some_event', :other_method
      def other_method; end
      def some_event; raise StandardError; end
    end

    klass.new(nil).__trigger('test.some_event')
  end

  it "should duplicate before events for children" do
    klass = Class.new(described_class) do
      orator_name 'test'

      before do; end
    end

    klass.before_list.length.should be 1

    subklass = Class.new(klass) do
      orator_name 'test2'
    end

    subklass.before_list.length.should be 1
    subklass.before_list.should eql(klass.before_list)
  end

  it "should duplicate after events for children" do
    klass = Class.new(described_class) do
      orator_name 'test'

      after do; end
    end

    klass.after_list.length.should be 1

    subklass = Class.new(klass) do
      orator_name 'test2'
    end

    subklass.after_list.length.should be 1
    subklass.after_list.should eql(klass.after_list)
  end

  it "should handle hash syntax" do
    klass = Class.new(described_class) do
      orator_name 'test'

      on('some.event' => :some_thing)
      def some_thing

      end
    end

    klass.event_list['some.event'].should eq([:some_thing])
  end
end
